library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity main is
	port (
				DATA     : in std_logic;
				CLK		: in std_logic;
				RESET    : in std_logic;
            RANDOMIZE : in std_logic; -- zapnuti generatoru nahodnych cisel
				POSITION : out std_logic_vector(3 downto 0);
				DISPLAY  : out std_logic_vector(7 downto 0)
         );
end main;

architecture Behavioral of main is
-------------------------------------------------------------
--
--    KOMPONENTY
--		==========
-------------------------------------------------------------
component zobrazovac is
	generic (number_of_segments : integer := 4);
	port (	CLK : in std_logic;
				DIGITS  : in std_logic_vector((number_of_segments * 4)-1 downto 0);
				POSITION : out std_logic_vector(number_of_segments-1 downto 0);
				DISPLAY  : out std_logic_vector(7 downto 0)
			);
end component;
-------------------------------------------------------------
component shift_reg is
	generic ( size : natural );
	port (
	  	CLK, RESET, CE : in std_logic;
		SERIAL_IN 	: in std_logic;
		SERIAL_OUT  : out std_logic;
		PARALEL_OUT : out std_logic_vector(size - 1 downto 0)
	);
end component;

-------------------------------------------------------------
  component clk_divisor is
   generic (divisor : natural );
   port (
      CLK_INPUT  : in std_logic;
      RESET      : in std_logic;
      CLK_OUTPUT : out std_logic
   );
   end component;
-------------------------------------------------------------
component crc_generic is
--	generic ( 
--					polynom : bit_vector(15 downto 0 ) := "1100000000000101"				
--				);
	port (
         data : in std_logic;
			reset : in std_logic;
			ce    : in std_logic;
         clk : in std_logic;
         output : out std_logic
     );
end component;
-------------------------------------------------------------
-------------------------------------------------------------
	signal numbers : std_logic_vector(15 downto 0);
	signal enable  : std_logic;
	signal data_wire : std_logic;
   signal input : std_logic;
begin
-------------------------------------------------------------
--
-- Propojeni
--
-------------------------------------------------------------
i_Zobrazovac : zobrazovac
	port map (
	  	CLK => CLK,
		DIGITS => numbers, -- vstup <= paralelni vystup z shift
                         -- registru
		POSITION => POSITION,
		DISPLAY => DISPLAY
	);
-------------------------------------------------------------
i_Clk_Divisor : clk_divisor
   generic map (divisor => 50_000_000 ) -- citani po 1 s
   port map (
      CLK_INPUT => CLK,
      RESET     => RESET,
      CLK_OUTPUT => enable -- rizeni pocitacni CRC a shifter
   );
-------------------------------------------------------------
i_CRC :	crc_generic
	 port map (  
		data => input, -- signal ovladany prepinacem RANDOMIZE
		reset => RESET,
		ce => enable,   
      clk => CLK,
      output => data_wire -- seriovy vstup <= bit. vystup z LSFR
	 );
-------------------------------------------------------------
i_Shifter: shift_reg 
	generic map (size => 16) -- 
	port map (
	  	CLK =>  CLK,
		RESET => RESET,
		CE => enable,
		SERIAL_IN => data_wire,
		SERIAL_OUT => open,
		PARALEL_OUT => numbers -- paralelni vystup => displej
	);	
-------------------------------------------------------------   
------------------------------------------------------------- 
   input <= not data_wire when RANDOMIZE = '1' else -- pri prepnuti
            -- prepinace na '1' generuje nahodna cisla
            DATA when RANDOMIZE = '0'; -- vstup dat na tlacitko
                    
end Behavioral;

